#!/bin/bash

OPTIND=1

while getopts "f:i:" opt; do
	case "$opt" in
	i)  PORT=$OPTARG
		;;
	f)  HEXFILE=$OPTARG
		;;
	esac
done

shift $((OPTIND-1))

[ "${1:-}" = "--" ] && shift

if [ -z ${PORT+x} ]; then
	echo no port given
	exit 4
fi

if [ -z ${HEXFILE+x} ]; then
	echo no file specified
	exit 5
fi

case "$(uname -s)" in
Darwin)
	DEV=/dev/cu.usbmodem
	STTYPARAM=-f
	AVRPATH=../darwin
	;;

Linux)
	DEV=/dev/ttyACM
	STTYPARAM=-F
	AVRPATH=../linux
	;;

esac

before=( $(ls $DEV* 2>/dev/null) )

found=0
timedout=1
timeout=0

stty $STTYPARAM $PORT 1200
if [ "$?" -ne "0" ]; then
	echo "port reset failed"
	exit 6
fi
sleep 0.4

while [ "$found" -eq 0 ]; do
	now=( $(ls $DEV* 2>/dev/null) )
	diff=${now[@]}
	for i in "${before[@]}"
	do
		diff=(${diff[@]/"$i"})
	done
	if [ ${#diff[@]} -ge 1 ]; then
		newport="${diff[0]}"
		found=1
		timedout=0
	else
		if [ "$timeout" -gt 40 ]; then
			timedout=1
			break;
		else
			(( timeout=timeout+1 ))
			sleep 0.25
		fi
	fi

	before=${now[@]}

	if [ "$timeout" -gt 20 ]; then
		for i in "${now[@]}"; do
			if [ "$i" == "$PORT" ] ; then
				newport="$PORT"
				found=1
				timedout=0
				break
			fi
		done
	fi
done

if [ "$timedout" -eq 1 ]; then
	echo "timed out"
	exit 3
fi

sleep 0.5

BINPATH="$( cd "$(dirname "$0")" ; pwd -P )"

chmod +x "$BINPATH"/"$AVRPATH"/avrdude
"$BINPATH"/"$AVRPATH"/avrdude -C"$BINPATH"/"$AVRPATH"/avrdude.conf -v -patmega32u4 -cavr109 -P"$newport" -b57600 -D -Uflash:w:"$HEXFILE":i 2>/dev/null

if [ "$?" -ne "0" ]; then
	echo "upload on port $newport failed"
	exit 1
fi
